(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* $Id: nsyntaxerr.n,v 1.2 2001/04/12 12:25:08 judaigi Exp $ *)

(* Auxiliary type for reporting syntax errors *)

signature SyntaxErrSig =
sig
type err_kinds = IllegalAsPat | IllegalLabel | IllegalField | IllegalPat
	       | IllegalRecBind | UnknownError | IllegalExpSeq
		   | IllegalType | IllegalTypeId | IllegalExp | IllegalVarId
		   | IllegalTypeVar | IllegalConId | IllegalDec | IllegalStrId
		   | IllegalFctId | IllegalSigId | IllegalStrExp | IllegalSigExp

type errmsg_kinds = EngGentle | HanGentle | HanTough

type error =
    Unclosed of Location.t * string * Location.t * string
  | Duptyid of Location.t * Location.t * string
  | Dupdatacon of Location.t * Location.t * string
  | Duppatvar of Location.t * Location.t * string
  | Orpaterr of Location.t
  | Predefined of Location.t * string
  | Other of Location.t * err_kinds

exception Error of error
exception Escape_error

val errmsg_type: errmsg_kinds ref
val report_error: error -> unit
end

structure NsyntaxErr : SyntaxErrSig = 
struct

open Format
open Location

type err_kinds = IllegalAsPat | IllegalLabel | IllegalField | IllegalPat
               | IllegalRecBind | UnknownError | IllegalExpSeq
			   | IllegalType | IllegalTypeId | IllegalExp | IllegalVarId
			   | IllegalTypeVar | IllegalConId | IllegalDec | IllegalStrId
			   | IllegalFctId | IllegalSigId | IllegalStrExp | IllegalSigExp

type errmsg_kinds = EngGentle | HanGentle | HanTough

val errmsg_type = ref HanGentle

type error =
    Unclosed of Location.t * string * Location.t * string
  | Duptyid of Location.t * Location.t * string
  | Dupdatacon of Location.t * Location.t * string
  | Duppatvar of Location.t * Location.t * string
  | Orpaterr of Location.t
  | Predefined of Location.t * string
  | Other of Location.t * err_kinds

exception Error of error
exception Escape_error

fun print_loc x = Location.print Format.std_formatter x

val msglist = 
  [(IllegalAsPat,
    "As pattern must be used with variable pattern.",
    "As Ͽ ׻  ϰ  մϴ.",
    "As տ ׻  ´ٴϱ!!"),
   (IllegalLabel,
    "Illegal record label.",
    "߸ ڵ ǥ(label)Դϴ.",
    "̻Ѱ ڲ ڵ ǥ , ڸ̴ ڸ̴ ؾ Ѵٴϱ!!"),
   (IllegalField,
    "Illegal field of record pattern.",
    "ڵ ʵ尡 ̻մϴ.",
    "ڵ ȿٰ ̻Ѱ ??"),
   (IllegalPat,
    "Illegal pattern.",
    "߸ Դϴ.",
    "̰͵ ̶ ??"),
   (IllegalRecBind,
    "Recursive variable must be bound to 'fn' expression.",
    " ǵ  ׻ 'fn' ǥ ; մϴ.",
    "let rec  ҷ, 'fn' Ŷ, ٸ     𸣴ϱ."),
   (IllegalExpSeq,
    "Sequence of expression must end with expression(Not semi-colon).",
    "α׷ ǥ  ׻ α׷ ǥ  մϴ(ݷ ȵ˴ϴ).",
    "α׷ ǥ ҷ   α׷ ǥ̾ Ѵٱ. ݷ  ɷ  ȵ."),
   (IllegalType,
    "This should be a proper type.",
	"ùٸ Ÿ ; մϴ.",
	"ùٸ Ÿ ; մϴ."),
   (IllegalTypeId,
    "This should be proper type identifier.",
	"ùٸ Ÿ ĺڰ ; մϴ.",
	"ùٸ Ÿ ĺڰ ; մϴ."),
   (IllegalExp,
    "This should be a proper expression.",
	"ùٸ ǥ ; մϴ.",
	"ùٸ ǥ ; մϴ."),
   (IllegalVarId,
    "This should be proper variable identifier.",
	"ùٸ  ĺڰ ; մϴ.",
	"ùٸ  ĺڰ ; մϴ."),
   (IllegalTypeVar,
    "This should be proper type variable.",
	"ùٸ Ÿ  ; մϴ.",
	"ùٸ Ÿ  ; մϴ."),
   (IllegalConId,
    "This should be proper constructor identifier.",
	"ùٸ  ĺڰ ; մϴ.",
	"ùٸ  ĺڰ ; մϴ."),
   (IllegalDec,
    "This should be proper declaration.",
	"ùٸ  ; մϴ.",
	"ùٸ  ; մϴ."),
   (IllegalStrId,
    "This should be proper structure identifer.",
	"ùٸ  ĺڰ ; մϴ.",
	"ùٸ  ĺڰ ; մϴ."),
   (IllegalFctId,
    "This should be proper functor identifer.",
	"ùٸ  ĺڰ ; մϴ.",
	"ùٸ  ĺڰ ; մϴ."),
   (IllegalSigId,
    "This should be proper signature identifier.",
	"ùٸ ñ׳ó ĺڰ ; մϴ.",
	"ùٸ ñ׳ó ĺڰ ; մϴ."),
   (IllegalStrExp,
    "This should be proper structure expressions.",
	"ùٸ  ǥ ; մϴ.",
	"ùٸ  ǥ ; մϴ."),
   (IllegalSigExp,
    "This should be proper signature expressions.",
	"ùٸ ñ׳ó ǥ ; մϴ.",
	"ùٸ ñ׳ó ǥ ; մϴ.")
   ]
  
fun print_err_kinds e =
  let val (k,msg1,msg2,msg3) =
            (List.find (fn (k,_,_,_) => k = e) msglist)
			handle Not_found =>
	        	(UnknownError,
             	 "Unknown error ocurred.",
             	 "ϴ  ߻߽ϴ.",
             	 "̰  𸣰ڴ.  ˾Ƽ ˾ƺ.")
   in print_string 
        (case !errmsg_type of
		   EngGentle => msg1
		 | HanGentle => msg2
		 | HanTough => msg3)
  end
  

val report_error = fn
    (Unclosed(opening_loc, opening, closing_loc, closing)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations opening_loc closing_loc
      then 
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: '"
	      | HanGentle => ": '"
	      | HanTough  => "û Ǽ: '");
         print_string closing;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' expected, the highlighted '"
	      | HanGentle => "' ()  ֽϴ. ǥõ '"
	      | HanTough  => "'  Գ!  ǥ '");
         print_string opening;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' might be unmatched"
	      | HanGentle => "' () ¦  ʴ  ϴ."
	      | HanTough  => "'  ȸ ̴. Ķ."))
      else
        (print_loc closing_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: '"
	      | HanGentle => ": '"
	      | HanTough  => "û Ǽ: '");
         print_string closing;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' expected "
	      | HanGentle => "' ()  ֽϴ. "
	      | HanTough  => "'  Ծ! "); 
	     force_newline();
         print_loc opening_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "This '"
	      | HanGentle => " ġ '"
	      | HanTough  => " ǥ '");
         print_string opening;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' might be unmatched"
	      | HanGentle => "' () ¦  ʴ  ϴ."
	      | HanTough  => "'  ȸ ̴. Ķ."))
  | (Duptyid(previous_loc, current_loc, name)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: type name '"
	      | HanGentle => ": Ÿ ̸ '"
	      | HanTough  => "û Ǽ: Ÿ ̸ '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' is duplicated at the highlighted positions"
	      | HanGentle => "' () ϳ Ÿǳ ι Ͽϴ. "
			     ^"ǥõ κ   ǵ κ Դϴ."
	      | HanTough  => "'  ǥ κп Ȱ  . ׷ ȵǿ~"))
      else
        (print_loc current_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: type name '"
	      | HanGentle => ": Ÿ ̸ '"
	      | HanTough  => "û Ǽ: Ÿ ̸ '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' previously appeared." 
	      | HanGentle => "'  ̹ ϳ Ÿǳ ̹ ǵǾϴ."
	      | HanTough  => "'  ̹ տ ֱ. ׷ ȵǿ~~");
	     force_newline();
         print_loc previous_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Previous position of the type name '"
	      | HanGentle => " ġ ̹ ǵǾ ֽϴ."
	      | HanTough  => "   ,  ϳ ٲ.");
	     if !errmsg_type = EngGentle
	     then ( print_string name; print_string "'" )
	     else ())
  | (Duppatvar(previous_loc, current_loc, name)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: pattern variable '"
	      | HanGentle => ":   '"
	      | HanTough  => "û Ǽ:   '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' is duplicated at the highlighted positions"
	      | HanGentle => "' ()  Ͽ ι ϴ. "^
	   		     "ǥõ  Ȯϼ"
	      | HanTough  => "'   Ͽ ι    ϶? "^
			     " ǥ  Ȯ ."))
      else
        (print_loc current_loc;
	     print_string
	     (case !errmsg_type of
	        EngGentle => "Syntax error: pattern variable '"
	      | HanGentle => ":   '"
	      | HanTough  => "û Ǽ:   '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' previously appeared."
	      | HanGentle => "' ()  ϳ ̹ ϴ."
	      | HanTough  => "'  ̹  ִ ̱. ");
	     force_newline();
	     print_loc previous_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Previous position of the pattern variable '"
	      | HanGentle => " ġ ̹ ǵǾ ֽϴ."
	      | HanTough  => "   ,  ϳ ٲ.");
	     if !errmsg_type = EngGentle
	     then ( print_string name; print_string "'" )
	     else ())
  | (Dupdatacon(previous_loc, current_loc, name)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: data constructor '"
	      | HanGentle => ": Ÿ  '"
	      | HanTough  => "û Ǽ: Ÿ  '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' is duplicated at the highlighted positions."
	      | HanGentle => "'   ǳ  ̻ ϴ."
	      | HanTough  => "'  ι    ϳ? ϳ ٲ."))
      else
        (print_loc current_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: data constructor '"
	      | HanGentle => ": Ÿ  '"
	      | HanTough  => "û Ǽ: Ÿ  '");
         print_string name;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "' previously appeared."
	      | HanGentle => "'   ǳ ̹ ̰ ֽϴ."
	      | HanTough  => "'  ̹ ǵ ε? -.-");
	     force_newline();
         print_loc previous_loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Previous position of the data constructor '"
	      | HanGentle => " ġ ̹ ǵǾ ֽϴ."
	      | HanTough  => "   ,  ϳ ٲ.");
         print_string name;
	     print_string "'")
  | (Orpaterr (loc)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations loc loc
      then
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: Or patterns must have same "^
	     		     "pattern variables"
	      | HanGentle => ": Or   ϵ  "^
	   		     " ƾ մϴ."
	      | HanTough  => "û Ǽ: Or ϵ鿡    "^
	   		     "ƾ Ѵٱ. ȱ׷ ̻ݾ?"))
      else
        (print_loc loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: Or patterns must have same "^
	     		     "pattern variables"
	      | HanGentle => ": Or   ϵ  "^
	   		     " ƾ մϴ."
	      | HanTough  => "û Ǽ: Or ϵ鿡    "^
	   		     "ƾ Ѵٱ. ȱ׷ ̻ݾ?"))
  | (Predefined (loc,name)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations loc loc
      then
        (print_string
         (case !errmsg_type of
            EngGentle => "Syntax error: reserved name "^name^
                     " can not be used."
          | HanGentle => ": ⺻ ǵ ̸ "^name^
                 "    ϴ."
          | HanTough  => "û Ǽ: "^name^
                 " ̹ صξ   ."))
      else
        (print_loc loc;
         print_string
         (case !errmsg_type of
            EngGentle => "Syntax error: reserved name "^name^
                     " can not be used."
          | HanGentle => ": ⺻ ǵ ̸ "^name^
                 "    ϴ."
          | HanTough  => "û Ǽ: "^name^
                 " ̹ صξ   ."))
  | (Other (loc,e)) =>
      if String.length !Location.input_name = 0
      && Location.highlight_locations loc loc
      then
        (print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: "
	      | HanGentle => ": "
	      | HanTough  => "û Ǽ: ");
         print_err_kinds e)
      else
        (print_loc loc;
         print_string 
	     (case !errmsg_type of
	        EngGentle => "Syntax error: "
	      | HanGentle => ": "
	      | HanTough  => "û Ǽ: ");
         print_err_kinds e)
end
